=== modified file 'breezy/_annotator_py.py'
--- breezy/_annotator_py.py	2019-03-02 21:46:18 +0000
+++ breezy/_annotator_py.py	2020-01-25 13:08:09 +0000
@@ -127,7 +127,7 @@
             if record.storage_kind == 'absent':
                 raise errors.RevisionNotPresent(record.key, self._vf)
             this_key = record.key
-            lines = osutils.chunks_to_lines(record.get_bytes_as('chunked'))
+            lines = record.get_bytes_as('lines')
             num_lines = len(lines)
             self._text_cache[this_key] = lines
             yield this_key, lines, num_lines

=== modified file 'breezy/bzr/groupcompress.py'
--- breezy/bzr/groupcompress.py	2019-06-15 22:30:37 +0000
+++ breezy/bzr/groupcompress.py	2020-01-25 13:08:09 +0000
@@ -466,7 +466,7 @@
                 return self._manager._wire_bytes()
             else:
                 return b''
-        if storage_kind in ('fulltext', 'chunked'):
+        if storage_kind in ('fulltext', 'chunked', 'lines'):
             if self._bytes is None:
                 # Grab and cache the raw bytes for this entry
                 # and break the ref-cycle with _manager since we don't need it
@@ -482,8 +482,10 @@
                 # refcycle here, but instead in manager.get_record_stream()
             if storage_kind == 'fulltext':
                 return self._bytes
-            else:
+            elif storage_kind == 'chunked':
                 return [self._bytes]
+            else:
+                return self._bytes.splitlines(True)
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 

=== modified file 'breezy/bzr/knit.py'
--- breezy/bzr/knit.py	2019-06-16 14:57:29 +0000
+++ breezy/bzr/knit.py	2020-01-25 13:08:09 +0000
@@ -273,8 +273,7 @@
             [compression_parent], 'unordered', True))
         if basis_entry.storage_kind == 'absent':
             raise errors.RevisionNotPresent(compression_parent, self._basis_vf)
-        basis_chunks = basis_entry.get_bytes_as('chunked')
-        basis_lines = osutils.chunks_to_lines(basis_chunks)
+        basis_lines = basis_entry.get_bytes_as('lines')
         # Manually apply the delta because we have one annotated content and
         # one plain.
         basis_content = PlainKnitContent(basis_lines, compression_parent)
@@ -309,8 +308,7 @@
             [compression_parent], 'unordered', True))
         if basis_entry.storage_kind == 'absent':
             raise errors.RevisionNotPresent(compression_parent, self._basis_vf)
-        basis_chunks = basis_entry.get_bytes_as('chunked')
-        basis_lines = osutils.chunks_to_lines(basis_chunks)
+        basis_lines = basis_entry.get_bytes_as('lines')
         basis_content = PlainKnitContent(basis_lines, compression_parent)
         # Manually apply the delta because we have one annotated content and
         # one plain.
@@ -381,19 +379,21 @@
                 self._create_network_bytes()
             return self._network_bytes
         if ('-ft-' in self.storage_kind
-                and storage_kind in ('chunked', 'fulltext')):
+                and storage_kind in ('chunked', 'fulltext', 'lines')):
             adapter_key = (self.storage_kind, 'fulltext')
             adapter_factory = adapter_registry.get(adapter_key)
             adapter = adapter_factory(None)
             bytes = adapter.get_bytes(self)
             if storage_kind == 'chunked':
                 return [bytes]
+            elif storage_kind == 'lines':
+                return bytes.splitlines(True)
             else:
                 return bytes
         if self._knit is not None:
             # Not redundant with direct conversion above - that only handles
             # fulltext cases.
-            if storage_kind == 'chunked':
+            if storage_kind in ('chunked', 'lines'):
                 return self._knit.get_lines(self.key[0])
             elif storage_kind == 'fulltext':
                 return self._knit.get_text(self.key[0])
@@ -435,9 +435,9 @@
                 # all the keys etc are contained in the bytes returned in the
                 # first record.
                 return b''
-        if storage_kind in ('chunked', 'fulltext'):
+        if storage_kind in ('chunked', 'fulltext', 'lines'):
             chunks = self._generator._get_one_work(self.key).text()
-            if storage_kind == 'chunked':
+            if storage_kind in ('chunked', 'lines'):
                 return chunks
             else:
                 return b''.join(chunks)
@@ -1747,9 +1747,8 @@
                         buffered = True
                 if not buffered:
                     self._index.add_records([index_entry])
-            elif record.storage_kind == 'chunked':
-                self.add_lines(record.key, parents,
-                               osutils.chunks_to_lines(record.get_bytes_as('chunked')))
+            elif record.storage_kind in ('chunked', 'file'):
+                self.add_lines(record.key, parents, record.get_bytes_as('lines'))
             else:
                 # Not suitable for direct insertion as a
                 # delta, either because it's not the right format, or this
@@ -2109,7 +2108,7 @@
         if key in self.nonlocal_keys:
             record = next(self.get_record_stream())
             # Create a content object on the fly
-            lines = osutils.chunks_to_lines(record.get_bytes_as('chunked'))
+            lines = record.get_bytes_as('lines')
             return PlainKnitContent(lines, record.key)
         else:
             # local keys we can ask for directly

=== modified file 'breezy/bzr/versionedfile.py'
--- breezy/bzr/versionedfile.py	2020-01-25 12:49:36 +0000
+++ breezy/bzr/versionedfile.py	2020-01-25 13:08:09 +0000
@@ -125,6 +125,8 @@
             return self._chunks
         elif storage_kind == 'fulltext':
             return b''.join(self._chunks)
+        elif storage_kind == 'lines':
+            return list(osutils.chunks_to_lines(self._chunks))
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
@@ -160,6 +162,8 @@
             return self._text
         elif storage_kind == 'chunked':
             return [self._text]
+        elif storage_kind == 'lines':
+            return self._text.splitlines(True)
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
@@ -188,6 +192,8 @@
             return self.file.read()
         elif storage_kind == 'chunked':
             return list(osutils.file_iterator(self.file))
+        elif storage_kind == 'lines':
+            return self.file.readlines()
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
@@ -1058,13 +1064,11 @@
                                   if not mpvf.has_version(p))
         # It seems likely that adding all the present parents as fulltexts can
         # easily exhaust memory.
-        chunks_to_lines = osutils.chunks_to_lines
         for record in self.get_record_stream(needed_parents, 'unordered',
                                              True):
             if record.storage_kind == 'absent':
                 continue
-            mpvf.add_version(chunks_to_lines(record.get_bytes_as('chunked')),
-                             record.key, [])
+            mpvf.add_version(record.get_bytes_as('lines'), record.key, [])
         for (key, parent_keys, expected_sha1, mpdiff), lines in zip(
                 records, mpvf.get_line_list(versions)):
             if len(parent_keys) == 1:

=== modified file 'breezy/bzr/vf_repository.py'
--- breezy/bzr/vf_repository.py	2020-01-20 00:03:54 +0000
+++ breezy/bzr/vf_repository.py	2020-01-25 13:08:09 +0000
@@ -1197,9 +1197,8 @@
         stream = self.inventories.get_record_stream(keys, 'unordered', True)
         for record in stream:
             if record.storage_kind != 'absent':
-                chunks = record.get_bytes_as('chunked')
+                lines = record.get_bytes_as('lines')
                 revid = record.key[-1]
-                lines = osutils.chunks_to_lines(chunks)
                 for line in lines:
                     yield line, revid
 

=== modified file 'breezy/bzr/weave.py'
--- breezy/bzr/weave.py	2019-03-02 21:46:18 +0000
+++ breezy/bzr/weave.py	2020-01-25 13:08:09 +0000
@@ -426,7 +426,7 @@
                     record.storage_kind == 'chunked'):
                 self.add_lines(
                     record.key[0], parents,
-                    osutils.chunks_to_lines(record.get_bytes_as('chunked')))
+                    record.get_bytes_as('lines'))
             else:
                 adapter_key = record.storage_kind, 'fulltext'
                 try:

=== modified file 'breezy/merge.py'
--- breezy/merge.py	2020-01-18 16:14:28 +0000
+++ breezy/merge.py	2020-01-25 13:08:09 +0000
@@ -2007,8 +2007,7 @@
         for record in self.vf.get_record_stream(keys, 'unordered', True):
             if record.storage_kind == 'absent':
                 raise errors.RevisionNotPresent(record.key, self.vf)
-            result[record.key[-1]] = osutils.chunks_to_lines(
-                record.get_bytes_as('chunked'))
+            result[record.key[-1]] = record.get_bytes_as('lines')
         return result
 
     def plan_merge(self):

